#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = ''
__author__ = 'HaiFeng'
__mtime__ = '2016/9/23'
"""

from ast import Lambda
import threading
import ctypes

from .quote_ctp import Quote
from .structs import *
from .structs_py import *

# CThostFtdcRspUserLoginField, CThostFtdcRspInfoField, CThostFtdcDepthMarketDataField, CThostFtdcSpecificInstrumentField


class CtpQuote(object):
    """"""

    def __init__(self):
        self.q = Quote()
        self.inst_tick = {}
        self.logined = False
        self.nRequestID = 0

    def ReqConnect(self, pAddress: str):
        """连接行情前置

        :param pAddress:
        """
        self.q.CreateApi()
        spi = self.q.CreateSpi()
        self.q.RegisterSpi(spi)

        self.q.OnFrontConnected = self._OnFrontConnected
        self.q.OnFrontDisconnected = self._OnFrontDisConnected
        self.q.OnRspUserLogin = self._OnRspUserLogin
        self.q.OnRtnDepthMarketData = self._OnRtnDepthMarketData
        self.q.OnRspSubMarketData = self._OnRspSubMarketData

        self.q.RegisterFront(pAddress)
        self.q.Init()

    def ReqUserLogout(self):
        """退出接口(正常退出,不会触发OnFrontDisconnected)"""
        self.q.Release()
        # 确保隔夜或重新登录时的第1个tick不被发送到客户端
        self.inst_tick.clear()
        self.logined = False
        threading.Thread(target=self.OnDisConnected, args=(self, 0)).start()

    def ReqUserLogin(self, user: str, pwd: str, broker: str):
        """登录

        :param user:
        :param pwd:
        :param broker:
        """
        f = CThostFtdcReqUserLoginField()
        f.BrokerID = bytes(broker, encoding="ascii")
        f.UserID = bytes(user, encoding="ascii")
        f.Password = bytes(pwd, encoding="ascii")
        f.UserProductInfo = bytes("@hf", encoding="ascii")
        self.nRequestID += 1
        self.q.ReqUserLogin(f, self.nRequestID)

    def ReqSubscribeMarketData(self, pInstrument: list):
        """订阅合约行情

        :param pInstrument: 行情列表
        """
        inst_p = (ctypes.c_char_p * len(pInstrument))()
        for x in range(len(pInstrument)):
            inst_p[x] = bytes(pInstrument[x], encoding="ascii")
        self.q.SubscribeMarketData(inst_p, len(pInstrument))

    def _OnFrontConnected(self):
        """"""
        threading.Thread(target=self.OnConnected, args=(self,)).start()

    def _OnFrontDisConnected(self, reason: int):
        """"""
        # 确保隔夜或重新登录时的第1个tick不被发送到客户端
        self.inst_tick.clear()
        self.logined = False
        threading.Thread(target=self.OnDisConnected, args=(self, reason)).start()

    def _OnRspUserLogin(
        self,
        pRspUserLogin: CThostFtdcRspUserLoginField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """"""
        info = InfoField()
        info.ErrorID = pRspInfo.getErrorID()
        info.ErrorMsg = pRspInfo.getErrorMsg()
        self.logined = True
        threading.Thread(target=self.OnUserLogin, args=(self, info)).start()

    def _OnRspSubMarketData(
        self,
        pSpecificInstrument: CThostFtdcSpecificInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        pass

    def _OnRtnDepthMarketData(self, pDepthMarketData: CThostFtdcDepthMarketDataField):
        """"""
        tick: Tick = None
        # 这个逻辑交由应用端处理更合理 ==> 第一个tick不送给客户端(以处理隔夜早盘时收到夜盘的数据的问题)
        inst = pDepthMarketData.getInstrumentID()
        if inst not in self.inst_tick:
            tick = Tick()
            self.inst_tick[inst] = tick
        else:
            tick = self.inst_tick[inst]

        tick.AskPrice = pDepthMarketData.getAskPrice1()
        tick.AskVolume = pDepthMarketData.getAskVolume1()
        tick.AveragePrice = pDepthMarketData.getAveragePrice()
        tick.BidPrice = pDepthMarketData.getBidPrice1()
        tick.BidVolume = pDepthMarketData.getBidVolume1()
        tick.Instrument = pDepthMarketData.getInstrumentID()
        tick.LastPrice = pDepthMarketData.getLastPrice()
        tick.OpenInterest = pDepthMarketData.getOpenInterest()
        tick.Volume = pDepthMarketData.getVolume()

        # 用tradingday替代Actionday不可取
        # day = pDepthMarketData.getTradingDay()
        # str = day + ' ' + pDepthMarketData.getUpdateTime()
        # if day is None or day == ' ':
        #     str = time.strftime('%Y%m%d %H:%M:%S', time.localtime())
        # tick.UpdateTime = str  # time.strptime(str, '%Y%m%d %H:%M:%S')

        tick.UpdateTime = pDepthMarketData.getUpdateTime()
        tick.UpdateMillisec = pDepthMarketData.getUpdateMillisec()
        tick.UpperLimitPrice = pDepthMarketData.getUpperLimitPrice()
        tick.LowerLimitPrice = pDepthMarketData.getLowerLimitPrice()
        tick.PreOpenInterest = pDepthMarketData.getPreOpenInterest()

        # 用线程会导入多数据入库时报错
        # threading.Thread(target=self.OnTick, args=(self, tick))
        self.OnTick(self, tick)

    def OnDisConnected(self, obj, error: int):
        """"""
        print(f"=== [QUOTE] OnDisConnected===\nerror: {str(error)}")

    def OnConnected(self, obj):
        """"""
        print("=== [QUOTE] OnConnected ===")

    def OnUserLogin(self, obj, info: InfoField):
        """"""
        print(f"=== [QUOTE] OnUserLogin ===\n{info}")

    def OnTick(self, obj, f: Tick):
        """"""
        print(f"=== [QUOTE] OnTick ===\n{f.__dict__}")


def connected(obj):
    print("connected")
    obj.ReqUserLogin("008105", "1", "9999")


def logged(obj, info):
    print(info)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
__title__ = ''
__author__ = 'HaiFeng'
__mtime__ = '2016/9/22'
"""

import threading
import itertools
import time
import os
import platform
from time import sleep

from .enums import (
    OrderType,
    InstrumentStatus,
    DirectType,
    OffsetType,
    HedgeType,
    TradeTypeType,
)
from .structs_py import *
from .trade_ctp import Trade
from .structs import *

# CThostFtdcInputOrderActionField, CThostFtdcInputOrderField, CThostFtdcInstrumentField, CThostFtdcInstrumentStatusField, CThostFtdcInvestorPositionField, CThostFtdcOrderField, CThostFtdcRspInfoField, CThostFtdcRspUserLoginField, CThostFtdcSettlementInfoConfirmField, CThostFtdcTradingAccountField, CThostFtdcTradingNoticeInfoField, CThostFtdcQuoteField, CThostFtdcInputQuoteField, CThostFtdcInputForQuoteField, CThostFtdcInvestorPositionDetailField, CThostFtdcRspAuthenticateField
from .datatype import *

# TThostFtdcActionFlagType, TThostFtdcClassTypeType, TThostFtdcContingentConditionType, TThostFtdcDirectionType, TThostFtdcOffsetFlagType, TThostFtdcForceCloseReasonType, TThostFtdcHedgeFlagType, TThostFtdcOrderPriceTypeType, TThostFtdcPosiDirectionType, TThostFtdcTimeConditionType, TThostFtdcTradingTypeType, TThostFtdcVolumeConditionType, TThostFtdcOrderStatusType, TThostFtdcInstrumentStatusType, TThostFtdcTradeTypeType, TThostFtdcAppTypeType


class CtpTrade:
    """"""

    def __init__(self, sys_name: str = "CTP"):
        self.front_address = ""
        self.investor = ""
        self.password = ""
        self.broker = ""
        self.pub_ip = ""
        self.port = ""
        self.logined = False
        self.tradingday = ""

        self.instruments = {}
        self.orders = {}
        self.trades = {}
        self.account: TradingAccount = None
        self.positions = {}
        self.instrument_status = {}

        self._session = ""
        self._orderid_sysid = {}
        self._posi = []

        self.t = Trade(sys_name)
        self.nRequestID = 0
        print(self.t.GetVersion())

    def getReqID(self):
        self.nRequestID += 1
        return self.nRequestID

    def strToBytes(self, msg: str):
        return bytes(msg, encoding="ascii")

    def enumToBytes(self, enumVar: Enum):
        return bytes(chr(enumVar.value), encoding="ascii")

    def _OnFrontConnected(self):
        threading.Thread(target=self.OnConnected, args=(self,)).start()

    def _OnFrontDisconnected(self, nReason):
        self.logined = False
        print(nReason)
        # 下午收盘后会不停断开再连接 4097错误
        if nReason == 4097 or nReason == 4098:
            threading.Thread(target=self._reconnect).start()
        else:
            threading.Thread(target=self.OnDisConnected, args=(self, nReason)).start()

    def _reconnect(self):
        if sum([1 if stat == "Continous" else 0 for exc, stat in self.instrument_status.items()]) == 0:
            print(time.strftime("%Y%m%d %H:%M:%S", time.localtime()))
            self.t.Release()
            time.sleep(600)
            self.ReqConnect(self.front_address)

    def _OnRspAuthenticate(
        self,
        pRspAuthenticateField: CThostFtdcRspAuthenticateField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        if pRspInfo.getErrorID() == 0:
            if pRspAuthenticateField.getAppType() == TThostFtdcAppTypeType.THOST_FTDC_APP_TYPE_InvestorRelay:
                self.t.RegisterUserSystemInfo(
                    self.broker,
                    self.investor,
                    0,
                    "",
                    self.pub_ip,
                    self.port,
                    time.strftime("%H:%M:%S", time.localtime()),
                )
            elif pRspAuthenticateField.getAppType() == TThostFtdcAppTypeType.THOST_FTDC_APP_TYPE_OperatorRelay:
                self.t.SubmitUserSystemInfo(
                    self.broker,
                    self.investor,
                    0,
                    "",
                    self.pub_ip,
                    self.port,
                    time.strftime("%H:%M:%S", time.localtime()),
                )
            f = CThostFtdcReqUserLoginField()
            f.BrokerID = self.strToBytes(self.broker)
            f.UserID = self.strToBytes(self.investor)
            f.Password = self.strToBytes(self.password)
            f.UserProductInfo = self.strToBytes("@hf")
            self.t.ReqUserLogin(f, self.getReqID())
        else:
            info = InfoField()
            info.ErrorID = pRspInfo.getErrorID()
            info.ErrorMsg = f"认证错误:{pRspInfo.getErrorMsg()}"
            threading.Thread(target=self.OnUserLogin, args=(self, info)).start()

    def _OnRspUserLogin(
        self,
        pRspUserLogin: CThostFtdcRspUserLoginField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """"""
        if pRspInfo.getErrorID() == 0:
            self.session = pRspUserLogin.getSessionID()
            self.tradingday = pRspUserLogin.getTradingDay()
            f = CThostFtdcSettlementInfoConfirmField()
            f.BrokerID = self.strToBytes(self.broker)
            f.InvestorID = self.strToBytes(self.investor)
            self.t.ReqSettlementInfoConfirm(f, self.getReqID())
        elif self.logined:
            threading.Thread(target=self._relogin).start()
        else:
            info = InfoField()
            info.ErrorID = pRspInfo.getErrorID()
            info.ErrorMsg = pRspInfo.getErrorMsg()
            threading.Thread(target=self.OnUserLogin, args=(self, info)).start()

    def _relogin(self):
        # 隔夜重连=>处理'初始化'错误
        time.sleep(60 * 10)
        f = CThostFtdcReqUserLoginField()
        f.BrokerID = self.strToBytes(self.broker)
        f.UserID = self.strToBytes(self.investor)
        f.Password = self.strToBytes(self.password)
        f.UserProductInfo = self.strToBytes("@hf")
        self.t.ReqUserLogin(f, self.getReqID())

    def _OnRspSettlementInfoConfirm(
        self,
        pSettlementInfoConfirm: CThostFtdcSettlementInfoConfirmField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        if not self.logined:
            time.sleep(0.5)
            f = CThostFtdcQryClassifiedInstrumentField()
            f.TradingType = TThostFtdcTradingTypeType.THOST_FTDC_TD_TRADE.value
            f.ClassType = TThostFtdcClassTypeType.THOST_FTDC_INS_ALL.value
            self.t.ReqQryClassifiedInstrument(f, self.getReqID())

    def _qry(self):
        """查询帐号相关信息"""
        # restart 模式, 待rtnorder 处理完毕后再进行查询,否则会造成position混乱
        ord_cnt = 0
        trd_cnt = 0
        while True:
            time.sleep(0.5)
            if len(self.orders) == ord_cnt and len(self.trades) == trd_cnt:
                break
            ord_cnt = len(self.orders)
            trd_cnt = len(self.trades)
        time.sleep(1.1)
        self.t.ReqQryInvestorPosition(CThostFtdcQryInvestorPositionField(), self.getReqID())
        time.sleep(1.1)
        self.t.ReqQryTradingAccount(CThostFtdcQryTradingAccountField(), self.getReqID())
        time.sleep(1.1)

        print("logged")
        self.logined = True
        info = InfoField()
        info.ErrorID = 0
        info.ErrorMsg = "正确"
        threading.Thread(target=self.OnUserLogin, args=(self, info)).start()
        # 调用Release后程序异常退出,但不报错误:接口断开了仍然调用了查询指令
        # while self.logined:
        """查询持仓与权益"""
        self.t.ReqQryInvestorPosition(CThostFtdcQryInvestorPositionField(), self.getReqID())

    def _OnRtnInstrumentStatus(self, pInstrumentStatus: CThostFtdcInstrumentStatusField):
        if pInstrumentStatus.getInstrumentID() == "":
            return
        status = InstrumentStatus.Continous
        if pInstrumentStatus.getInstrumentStatus() == TThostFtdcInstrumentStatusType.THOST_FTDC_IS_Continous:
            status = InstrumentStatus.Continous
        elif pInstrumentStatus.getInstrumentStatus() == TThostFtdcInstrumentStatusType.THOST_FTDC_IS_Closed:
            status = InstrumentStatus.Closed
        elif str(pInstrumentStatus.getInstrumentStatus()).startswith("Auction"):
            status = InstrumentStatus.Auction
        else:
            status = InstrumentStatus.NoTrading
        self.instrument_status[pInstrumentStatus.getInstrumentID()] = status
        self.OnInstrumentStatus(self, pInstrumentStatus.getInstrumentID(), status)

    def _OnRspQryInstrument(
        self,
        pInstrument: CThostFtdcInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """"""
        inst = InstrumentField()
        inst.InstrumentID = pInstrument.getInstrumentID()
        inst.InstrumentName = pInstrument.getInstrumentName()
        inst.ProductID = pInstrument.getProductID()
        inst.ExchangeID = pInstrument.getExchangeID()
        inst.VolumeMultiple = pInstrument.getVolumeMultiple()
        inst.PriceTick = pInstrument.getPriceTick()
        inst.MaxOrderVolume = pInstrument.getMaxLimitOrderVolume()
        inst.ProductType = pInstrument.getProductClass()  # ProductClassType.Futures -> Futures
        self.instruments[inst.InstrumentID] = inst
        if bIsLast:
            sleep(1.1)
            """查询合约/持仓/权益"""
            print("start qry thread")
            threading.Thread(target=self._qry).start()  # 开启查询

    def _OnRspQryClassifiedInstrument(
        self,
        pInstrument: CThostFtdcInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """"""
        self._OnRspQryInstrument(pInstrument, pRspInfo, nRequestID, bIsLast)

    def _OnRspQryPosition(
        self,
        pInvestorPosition: CThostFtdcInvestorPositionField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """"""
        if pInvestorPosition.getInstrumentID() != "":  # 偶尔出现NULL的数据导致数据转换错误
            if pInvestorPosition.getInstrumentID() in self.instruments:  # 解决交易所自主合成某些不可交易的套利合约的问题如 SPC y2005&p2001
                self._posi.append(pInvestorPosition)  # Struct(**f.__dict__)) #dict -> object

        if bIsLast:
            # 先排序再group才有效
            self._posi = sorted(
                self._posi,
                key=lambda c: "{0}_{1}".format(
                    c.getInstrumentID(),
                    (DirectType.Buy if c.getPosiDirection() == TThostFtdcPosiDirectionType.THOST_FTDC_PD_Long else DirectType.Sell),
                ),
            )
            # direction需从posidiction转换为dictiontype
            for key, group in itertools.groupby(
                self._posi,
                lambda c: "{0}_{1}".format(
                    c.getInstrumentID(),
                    ("Buy" if c.getPosiDirection() == TThostFtdcPosiDirectionType.THOST_FTDC_PD_Long else "Sell"),
                ),
            ):
                pf = self.positions.get(key)
                if not pf:
                    pf = PositionField()
                    self.positions[key] = pf
                pf.Position = 0
                pf.TdPosition = 0
                pf.YdPosition = 0
                pf.CloseProfit = 0
                pf.PositionProfit = 0
                pf.Commission = 0
                pf.Margin = 0
                pf.Price = 0
                cost = 0.0
                for g in group:
                    if not pf.InstrumentID:
                        pf.InstrumentID = g.getInstrumentID()
                        pf.Direction = DirectType.Buy if g.getPosiDirection() == TThostFtdcPosiDirectionType.THOST_FTDC_PD_Long else DirectType.Sell
                    pf.Position += g.getPosition()
                    pf.TdPosition += g.getTodayPosition()
                    pf.YdPosition = pf.Position - pf.TdPosition
                    pf.CloseProfit += g.getCloseProfit()
                    pf.PositionProfit += g.getPositionProfit()
                    pf.Commission += g.getCommission()
                    pf.Margin += g.getUseMargin()
                    cost += g.OpenCost
                # pf.Position <= 0 ? 0 : (g.Sum(n => n.PositionCost) / DicInstrumentField[pf.InstrumentID].VolumeMultiple / pf.Position);
                if pf.InstrumentID in self.instruments:
                    vm = self.instruments[pf.InstrumentID].VolumeMultiple
                    pf.Price = 0 if pf.Position <= 0 else (cost / (vm if vm > 0 else 1) / pf.Position)
            self._posi.clear()

            """查询持仓与权益"""
            if not self.logined:
                return
            time.sleep(1.1)
            self.t.ReqQryTradingAccount(CThostFtdcQryTradingAccountField(), self.getReqID())

    def _OnRspQryPositionDetail(
        self,
        pInvestorPositionDetail: CThostFtdcInvestorPositionDetailField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """持仓明细"""
        if pInvestorPositionDetail.getInstrumentID() == "":
            return
        detail = PositionDetail()
        detail.Instrument = pInvestorPositionDetail.getInstrumentID()
        detail.CloseProfit = pInvestorPositionDetail.getCloseProfitByTrade()
        detail.Direction = DirectType.Buy if pInvestorPositionDetail.getDirection() == TThostFtdcDirectionType.THOST_FTDC_D_Buy else DirectType.Sell
        detail.HedgeFlag = HedgeType(list(TThostFtdcHedgeFlagType).index(pInvestorPositionDetail.getHedgeFlag()))
        detail.OpenDate = pInvestorPositionDetail.getOpenDate()
        detail.PositionProfit = pInvestorPositionDetail.getPositionProfitByTrade()
        detail.OpenPrice = pInvestorPositionDetail.getOpenPrice()
        detail.TradeType = TradeTypeType(list(TThostFtdcTradeTypeType).index(pInvestorPositionDetail.getTradeType()))
        detail.Volume = pInvestorPositionDetail.getVolume()
        self.position_details[pInvestorPositionDetail.getTradeID()] = detail

    def _OnRspQryAccount(
        self,
        pTradingAccount: CThostFtdcTradingAccountField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """"""
        if not self.account:
            self.account = TradingAccount()
        self.account.Available = pTradingAccount.getAvailable()
        self.account.CloseProfit = pTradingAccount.getCloseProfit()
        self.account.Commission = pTradingAccount.getCommission()
        self.account.CurrMargin = pTradingAccount.getCurrMargin()
        self.account.FrozenCash = pTradingAccount.getFrozenCash()
        self.account.PositionProfit = pTradingAccount.getPositionProfit()
        self.account.PreBalance = pTradingAccount.getPreBalance() + pTradingAccount.getDeposit() - pTradingAccount.getWithdraw()
        self.account.Fund = self.account.PreBalance + pTradingAccount.getCloseProfit() + pTradingAccount.getPositionProfit() - pTradingAccount.getCommission()
        self.account.Risk = 0 if self.account.Fund == 0 else self.account.CurrMargin / self.account.Fund
        if bIsLast:
            """查询持仓与权益"""
            if not self.logined:
                return
            time.sleep(1.1)
            self.t.ReqQryInvestorPosition(CThostFtdcQryInvestorPositionField(), self.getReqID())

    def _OnRtnOrder(self, pOrder: CThostFtdcOrderField):
        """"""
        id = "{0}|{1}|{2}".format(pOrder.getSessionID(), pOrder.getFrontID(), pOrder.getOrderRef())
        of = self.orders.get(id)
        if not of:
            of = OrderField()
            if pOrder.getOrderRef().isdigit():
                of.Custom = int(pOrder.getOrderRef()) % 1000000
            of.InstrumentID = pOrder.getInstrumentID()
            of.ExchangeID = pOrder.getExchangeID()
            of.InsertTime = pOrder.getInsertTime()
            of.Direction = DirectType.Buy if pOrder.getDirection() == TThostFtdcDirectionType.THOST_FTDC_D_Buy else DirectType.Sell
            of.Offset = (
                OffsetType.Open
                if pOrder.getCombOffsetFlag() == TThostFtdcOffsetFlagType.THOST_FTDC_OF_Open
                else (OffsetType.CloseToday if pOrder.getCombOffsetFlag() == TThostFtdcOffsetFlagType.THOST_FTDC_OF_CloseToday else OffsetType.Close)
            )
            of.Status = OrderStatus.Normal
            of.StatusMsg = pOrder.getStatusMsg()
            of.IsLocal = pOrder.getSessionID() == self.session
            of.LimitPrice = pOrder.getLimitPrice()
            of.OrderID = id
            of.Volume = pOrder.getVolumeTotalOriginal()
            of.VolumeLeft = of.Volume
            self.orders[id] = of
            threading.Thread(target=self.OnOrder, args=(self, of)).start()
        elif pOrder.getOrderStatus() == TThostFtdcOrderStatusType.THOST_FTDC_OST_Canceled:
            of.Status = OrderStatus.Canceled
            of.StatusMsg = pOrder.getStatusMsg()

            if of.StatusMsg.find("被拒绝") >= 0:
                info = InfoField()
                info.ErrorID = -1
                info.ErrorMsg = of.StatusMsg
                threading.Thread(target=self.OnErrOrder, args=(self, of, info)).start()
            else:
                threading.Thread(target=self.OnCancel, args=(self, of)).start()
        else:
            if pOrder.getOrderSysID():
                of.SysID = pOrder.getOrderSysID()
                self._orderid_sysid[pOrder.getOrderSysID()] = id  # 记录sysid与orderid关联,方便Trade时查找处理

    def _OnRtnTrade(self, f):
        """"""
        tf = TradeField()
        tf.Direction = DirectType.Buy if f.getDirection() == TThostFtdcDirectionType.THOST_FTDC_D_Buy else DirectType.Sell
        tf.ExchangeID = f.getExchangeID()
        tf.InstrumentID = f.getInstrumentID()
        tf.Offset = (
            OffsetType.Open
            if f.getOffsetFlag() == TThostFtdcOffsetFlagType.THOST_FTDC_OF_Open
            else (OffsetType.Close if f.getOffsetFlag() == TThostFtdcOffsetFlagType.THOST_FTDC_OF_Close else OffsetType.CloseToday)
        )
        tf.Price = f.getPrice()
        tf.SysID = f.getOrderSysID()
        tf.TradeID = f.getTradeID()
        tf.TradeTime = f.getTradeTime()
        tf.TradingDay = f.getTradingDay()
        tf.Volume = f.getVolume()

        self.trades[f"{tf.TradeID}{tf.Direction.value}"] = tf

        id = self._orderid_sysid[tf.SysID]
        of = self.orders[id]
        tf.OrderID = id  # tradeid 与 orderid 关联
        of.TradeTime = tf.TradeTime
        of.AvgPrice = (of.AvgPrice * (of.Volume - of.VolumeLeft) + tf.Price * tf.Volume) / (of.Volume - of.VolumeLeft + tf.Volume)
        of.TradeVolume = tf.Volume
        of.VolumeLeft -= tf.Volume
        if of.VolumeLeft == 0:
            of.Status = OrderStatus.Filled
            of.StatusMsg = "全部成交"
        else:
            of.Status = OrderStatus.Partial
            of.StatusMsg = "部分成交"
        # 更新持仓 *****
        if tf.Offset == OffsetType.Open:
            key = "{0}_{1}".format(tf.InstrumentID, "Buy" if tf.Direction == DirectType.Buy else "Sell")
            pf = self.positions.get(key)
            if not pf:
                pf = PositionField()
                self.positions[key] = pf
            pf.InstrumentID = tf.InstrumentID
            pf.Direction = tf.Direction
            pf.Price = (pf.Price * pf.Position + tf.Price * tf.Volume) / (pf.Position + tf.Volume)
            pf.TdPosition += tf.Volume
            pf.Position += tf.Volume
        else:
            key = "{0}_{1}".format(tf.InstrumentID, "Sell" if tf.Direction == DirectType.Buy else "Buy")
            pf: PositionField = self.positions.get(key)
            if pf:  # 有可能出现无持仓的情况
                if tf.Offset == OffsetType.CloseToday:
                    pf.TdPosition -= tf.Volume
                else:
                    ydclose = min(pf.YdPosition, tf.Volume)
                    if pf.YdPosition > 0:
                        pf.YdPosition -= ydclose
                    pf.TdPosition -= max(0, tf.Volume - ydclose)
                pf.Position -= tf.Volume
        threading.Thread(target=self._onRtn, args=(of, tf)).start()

    def _onRtn(self, of, tf):
        self.OnOrder(self, of)
        self.OnTrade(self, tf)

    def _OnRspOrder(
        self,
        pInputOrder: CThostFtdcInputOrderField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """"""
        info = InfoField()
        info.ErrorID = pRspInfo.getErrorID()
        info.ErrorMsg = pRspInfo.getErrorMsg()

        id = "{0}|{1}|{2}".format(self.session, "0", pInputOrder.getOrderRef())
        of = self.orders.get(id)
        if not of:
            of = OrderField()
            l = int(pInputOrder.getOrderRef())
            of.Custom = l % 1000000
            of.InstrumentID = pInputOrder.getInstrumentID()
            of.InsertTime = time.strftime("%H:%M:%S", time.localtime())
            # 对direction需特别处理（具体见ctp_struct）
            of.Direction = DirectType.Buy if pInputOrder.getDirection() == TThostFtdcDirectionType.THOST_FTDC_D_Buy else DirectType.Sell
            of.Offset = (
                OffsetType.Open
                if pInputOrder.getCombOffsetFlag() == TThostFtdcOffsetFlagType.THOST_FTDC_OF_Open
                else (OffsetType.CloseToday if pInputOrder.getCombOffsetFlag() == TThostFtdcOffsetFlagType.THOST_FTDC_OF_CloseToday else OffsetType.Close)
            )
            # of.Status = OrderStatus.Normal
            # of.StatusMsg = f.getStatusMsg()
            of.IsLocal = True
            of.LimitPrice = pInputOrder.getLimitPrice()
            of.OrderID = id
            of.Volume = pInputOrder.getVolumeTotalOriginal()
            of.VolumeLeft = of.Volume
            self.orders[id] = of

        of.Status = OrderStatus.Error
        of.StatusMsg = "{0}:{1}".format(info.ErrorID, info.ErrorMsg)
        threading.Thread(target=self.OnErrOrder, args=(self, of, info)).start()

    def _OnErrOrder(self, pInputOrder: CThostFtdcInputOrderField, pRspInfo: CThostFtdcRspInfoField):
        """"""
        id = "{0}|{1}|{2}".format(self.session, "0", pInputOrder.getOrderRef())
        of = self.orders.get(id)

        info = InfoField()
        info.ErrorID = pRspInfo.getErrorID()
        info.ErrorMsg = pRspInfo.getErrorMsg()

        if of and of.IsLocal:
            of.Status = OrderStatus.Error
            of.StatusMsg = "{0}:{1}".format(pRspInfo.getErrorID(), pRspInfo.getErrorMsg())
            threading.Thread(target=self.OnErrOrder, args=(self, of, info)).start()

    def _OnRspOrderAction(
        self,
        pInputOrderAction: CThostFtdcInputOrderActionField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        id = "{0}|{1}|{2}".format(
            pInputOrderAction.getSessionID(),
            pInputOrderAction.getFrontID(),
            pInputOrderAction.getOrderRef(),
        )
        if self.logined and id in self.orders:
            info = InfoField()
            info.ErrorID = pRspInfo.ErrorID
            info.ErrorMsg = pRspInfo.ErrorMsg
            threading.Thread(target=self.OnErrCancel, args=(self, self.orders[id], info)).start()

    def _OnRtnNotice(self, pTradingNoticeInfo: CThostFtdcTradingNoticeInfoField):
        """交易提醒"""
        msg = pTradingNoticeInfo.getFieldContent()
        if len(msg) > 0:
            threading.Thread(
                target=self.OnRtnNotice,
                args=(self, pTradingNoticeInfo.getSendTime(), msg),
            ).start()

    def _OnRtnQuote(self, pQuote: CThostFtdcQuoteField):
        threading.Thread(target=self.OnRtnQuote, args=(self, pQuote)).start()

    def _OnErrRtnQuote(self, pInputQuote: CThostFtdcInputQuoteField, pRspInfo: CThostFtdcRspInfoField):
        info = InfoField()
        info.ErrorID = pRspInfo.getErrorID()
        info.ErrorMsg = pRspInfo.getErrorMsg()
        threading.Thread(target=self.OnErrRtnQuote, args=(self, pInputQuote, info)).start()

    def _OnErrForQuoteInsert(
        self,
        pInputForQuote: CThostFtdcInputForQuoteField,
        pRspInfo: CThostFtdcRspInfoField,
    ):
        info = InfoField()
        info.ErrorID = pRspInfo.getErrorID()
        info.ErrorMsg = pRspInfo.getErrorMsg()
        threading.Thread(target=self.OnErrRtnForQuoteInsert, args=(self, pInputForQuote, info)).start()

    def _OnRspError(self, pRspInfo: CThostFtdcRspInfoField, nRequestID: int, bIsLast: bool):
        info = InfoField()
        info.ErrorID = pRspInfo.getErrorID()
        info.ErrorMsg = pRspInfo.getErrorMsg()
        threading.Thread(target=self.OnRspError, args=(self, info)).start()

    def GetVersion(self):
        return self.t.GetVersion()

    def ReqConnect(self, front: str):
        """连接交易前置

        :param front:
        """
        self.t.CreateApi()
        spi = self.t.CreateSpi()
        self.t.RegisterSpi(spi)

        self.t.OnFrontConnected = self._OnFrontConnected
        self.t.OnRspUserLogin = self._OnRspUserLogin
        self.t.OnRspAuthenticate = self._OnRspAuthenticate
        self.t.OnFrontDisconnected = self._OnFrontDisconnected
        # self.t.OnRspUserLogout = self._OnRspUserLogout
        self.t.OnRspSettlementInfoConfirm = self._OnRspSettlementInfoConfirm
        self.t.OnRtnOrder = self._OnRtnOrder
        self.t.OnRtnTrade = self._OnRtnTrade
        self.t.OnRspOrderInsert = self._OnRspOrder
        self.t.OnErrRtnOrderInsert = self._OnErrOrder
        self.t.OnRspOrderAction = self._OnRspOrderAction
        self.t.OnRtnInstrumentStatus = self._OnRtnInstrumentStatus
        self.t.OnRspQryInstrument = self._OnRspQryInstrument
        self.t.OnRspQryClassifiedInstrument = self._OnRspQryClassifiedInstrument
        self.t.OnRspQryTradingAccount = self._OnRspQryAccount
        self.t.OnRspQryInvestorPosition = self._OnRspQryPosition
        self.t.OnRspQryInvestorPositionDetail = self._OnRspQryPositionDetail
        self.t.OnRtnTradingNotice = self._OnRtnNotice
        self.t.OnRtnQuote = self._OnRtnQuote
        self.t.OnErrRtnQuoteInsert = self._OnErrRtnQuote
        self.t.OnErrRtnForQuoteInsert = self._OnErrForQuoteInsert
        self.t.OnRspError = self._OnRspError

        self.front_address = front

        self.t.RegisterFront(front)
        self.t.SubscribePrivateTopic(0)  # restart 同步处理order trade
        self.t.SubscribePublicTopic(0)
        self.t.Init()
        # self.t.Join()

    def ReqUserLogin(self, user: str, pwd: str, broker: str, appid: str, auth_code: str):
        """登录

        :param user:
        :param pwd:
        :param broker:
        """
        self.broker = broker
        self.investor = user
        self.password = pwd
        f = CThostFtdcReqAuthenticateField()
        f.UserID = self.strToBytes(user)
        f.AppID = self.strToBytes(appid)
        f.BrokerID = self.strToBytes(broker)
        f.AuthCode = self.strToBytes(auth_code)
        print(f.getBrokerID(), f.getAppID(), f.getAuthCode())
        self.t.ReqAuthenticate(f, self.getReqID())

    def ReqOrderInsert(
        self,
        pInstrument: str,
        pDirection: DirectType,
        pOffset: OffsetType,
        pPrice: float = 0.0,
        pVolume: int = 1,
        pType: OrderType = OrderType.Limit,
        pCustom: int = 0,
    ):
        """委托

        :param pInstrument:
        :param pDirection:
        :param pOffset:
        :param pPrice:
        :param pVolume:
        :param pType:
        :param pCustom:
        :return:
        """
        OrderPriceType = TThostFtdcOrderPriceTypeType.THOST_FTDC_OPT_AnyPrice
        TimeCondition = TThostFtdcTimeConditionType.THOST_FTDC_TC_IOC
        LimitPrice = 0.0
        VolumeCondition = TThostFtdcVolumeConditionType.THOST_FTDC_VC_AV

        if pType == OrderType.Market:  # 市价
            OrderPriceType = TThostFtdcOrderPriceTypeType.THOST_FTDC_OPT_AnyPrice
            TimeCondition = TThostFtdcTimeConditionType.THOST_FTDC_TC_IOC
            LimitPrice = 0.0
            VolumeCondition = TThostFtdcVolumeConditionType.THOST_FTDC_VC_AV
        elif pType == OrderType.Limit:  # 限价
            OrderPriceType = TThostFtdcOrderPriceTypeType.THOST_FTDC_OPT_LimitPrice
            TimeCondition = TThostFtdcTimeConditionType.THOST_FTDC_TC_GFD
            LimitPrice = pPrice
            VolumeCondition = TThostFtdcVolumeConditionType.THOST_FTDC_VC_AV
        elif pType == OrderType.FAK:  # FAK
            OrderPriceType = TThostFtdcOrderPriceTypeType.THOST_FTDC_OPT_LimitPrice
            TimeCondition = TThostFtdcTimeConditionType.THOST_FTDC_TC_IOC
            LimitPrice = pPrice
            VolumeCondition = TThostFtdcVolumeConditionType.THOST_FTDC_VC_AV
        elif pType == OrderType.FOK:  # FOK
            OrderPriceType = TThostFtdcOrderPriceTypeType.THOST_FTDC_OPT_LimitPrice
            TimeCondition = TThostFtdcTimeConditionType.THOST_FTDC_TC_IOC
            LimitPrice = pPrice
            VolumeCondition = TThostFtdcVolumeConditionType.THOST_FTDC_VC_CV  # 全部数量

        f = CThostFtdcInputOrderField()
        f.BrokerID = self.strToBytes(self.broker)
        f.InvestorID = self.strToBytes(self.investor)
        f.InstrumentID = self.strToBytes(pInstrument)
        f.OrderRef = self.strToBytes("%06d%06d" % (self.getReqID(), pCustom % 1000000))
        f.UserID = self.strToBytes(self.investor)
        f.ExchangeID = self.strToBytes(self.instruments[pInstrument].ExchangeID)
        # 此处ctp_enum与at_struct名称冲突
        f.Direction = TThostFtdcDirectionType.THOST_FTDC_D_Buy.value if pDirection == DirectType.Buy else TThostFtdcDirectionType.THOST_FTDC_D_Sell.value
        f.CombOffsetFlag = self.enumToBytes(
            TThostFtdcOffsetFlagType.THOST_FTDC_OF_Open
            if pOffset == OffsetType.Open
            else (TThostFtdcOffsetFlagType.THOST_FTDC_OF_CloseToday if pOffset == OffsetType.CloseToday else TThostFtdcOffsetFlagType.THOST_FTDC_OF_Close)
        )
        f.CombHedgeFlag = self.enumToBytes(TThostFtdcHedgeFlagType.THOST_FTDC_HF_Speculation)
        f.IsAutoSuspend = 0
        f.ForceCloseReason = TThostFtdcForceCloseReasonType.THOST_FTDC_FCC_NotForceClose.value
        f.IsSwapOrder = 0
        f.ContingentCondition = TThostFtdcContingentConditionType.THOST_FTDC_CC_Immediately.value
        f.VolumeCondition = VolumeCondition.value
        f.MinVolume = 1
        f.VolumeTotalOriginal = pVolume
        f.OrderPriceType = OrderPriceType.value
        f.TimeCondition = TimeCondition.value
        f.LimitPrice = LimitPrice
        self.t.ReqOrderInsert(f, self.getReqID())

    def ReqOrderAction(self, OrderID: str):
        """撤单

        :param OrderID:
        """
        of = self.orders[OrderID]

        if not of:
            return -1
        else:
            pOrderId = of.OrderID
            f = CThostFtdcInputOrderActionField()
            f.BrokerID = self.strToBytes(self.broker)
            f.InvestorID = self.strToBytes(self.investor)
            f.OrderRef = self.strToBytes(pOrderId.split("|")[2])
            f.FrontID = int(pOrderId.split("|")[1])
            f.SessionID = int(pOrderId.split("|")[0])
            f.InstrumentID = self.strToBytes(of.InstrumentID)
            f.ExchangeID = self.strToBytes(of.ExchangeID)
            f.ActionFlag = TThostFtdcActionFlagType.THOST_FTDC_AF_Delete.value

            return self.t.ReqOrderAction(f, self.getReqID())

    def ReqUserLogout(self):
        """退出接口"""
        self.logined = False
        time.sleep(3)
        f = CThostFtdcUserLogoutField()
        f.BrokerID = self.strToBytes(self.broker)
        f.InvestorID = self.strToBytes(self.investor)
        self.t.ReqUserLogout(f, self.getReqID())
        self.t.RegisterSpi(None)
        self.t.Release()
        threading.Thread(target=self.OnDisConnected, args=(self, 0)).start()

    def OnConnected(self, obj):
        """接口连接

        :param obj:
        """
        print("=== [TRADE] OnConnected ===".format(""))

    def OnDisConnected(self, obj, reason: int):
        """接口断开

        :param obj:
        :param reason:
        """
        print("=== [TRADE] OnDisConnected === \nreason: {0}".format(reason))

    def OnUserLogin(self, obj, info: InfoField):
        """登录响应

        :param obj:
        :param info:
        """
        print("=== [TRADE] OnUserLogin === \n{0}".format(info))

    def OnOrder(self, obj, f: OrderField):
        """委托响应

        :param obj:
        :param f:
        """
        print("=== [TRADE] OnOrder === \n{0}".format(f.__dict__))

    def OnTrade(self, obj, f: TradeField):
        """成交响应

        :param obj:
        :param f:
        """
        print("=== [TRADE] OnTrade === \n{0}".format(f.__dict__))

    def OnCancel(self, obj, f: OrderField):
        """
        撤单响应
            :param self:
            :param obj:
            :param f:OrderField:
        """
        print("=== [TRADE] OnCancel === \n{0}".format(f.__dict__))

    def OnErrCancel(self, obj, f: OrderField, info: InfoField):
        """
        撤单失败
            :param self:
            :param obj:
            :param f:OrderField:
            :param info:InfoField:
        """
        print("=== [TRADE] OnErrCancel ===\n{0}".format(f.__dict__))
        print(info)

    def OnRspError(self, obj, info: InfoField):
        """
        撤单失败
            :param self:
            :param obj:
            :param f:OrderField:
            :param info:InfoField:
        """
        print("=== [TRADE] OnRspError ===\n{0}".format(info.__dict__))

    def OnErrOrder(self, obj, f: OrderField, info: InfoField):
        """
        委托错误
            :param self:
            :param obj:
            :param f:OrderField:
            :param info:InfoField:
        """
        print("=== [TRADE] OnErrOrder ===\n{0}".format(f.__dict__))
        print(info)

    def OnInstrumentStatus(self, obj, inst: str, status: InstrumentStatus):
        """
        交易状态
            :param self:
            :param obj:
            :param inst:str:
            :param status:InstrumentStatus:
        """
        print("{}:{}".format(inst, str(status).strip().split(".")[-1]))

    def OnRtnNotice(self, obj, time: str, msg: str):
        """交易提醒

        :param obj:
        :param time:
        :param msg:
        :return:
        """
        print(f"=== OnRtnNotice===\n {time}:{msg}")

    def OnRtnQuote(self, obj, quote: CThostFtdcQuoteField):
        """报价通知

        :param obj:
        :param quote:
        :return:
        """
        print("=== [TRADE] OnRtnQuote ===\n{0}".format(quote.__dict__))

    def OnErrRtnQuote(self, obj, quote: CThostFtdcInputQuoteField, info: InfoField):
        """

        :param obj:
        :param quote:
        :return:
        """
        print("=== [TRADE] OnErrRtnQuote ===\n{0}".format(quote.__dict__))
        print(info)

    def OnErrRtnForQuoteInsert(self, obj, quote: CThostFtdcInputQuoteField, info: InfoField):
        """询价录入错误回报

        :param obj:
        :param quote:
        :return:
        """
        print("=== [TRADE] OnErrRtnForQuoteInsert ===\n{0}".format(quote.__dict__))
        print(info)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = ''
__author__ = 'HaiFeng'
__mtime__ = '2017/10/30'
"""

from enum import Enum


class TradeTypeType(Enum):
    """成交类型类型"""

    SplitCombinatio = 110
    """组合持仓拆分为单一持仓,初始化不应包含该类型的持仓"""
    Common = 48
    """普通成交"""
    OptionsExecution = 49
    """期权执行"""
    OTC = 50
    """OTC成交"""
    EFPDerived = 51
    """期转现衍生成交"""
    CombinationDerived = 52
    """组合衍生成交"""


class HedgeType(Enum):
    """投保"""

    Speculation = 0
    """投机"""
    Arbitrage = 1
    """套利"""
    Hedge = 2
    """套保"""
    MarketMaker = 3
    """做市商"""

    def __int__(self):
        """Enum"""
        return self.value


class DirectType(Enum):
    """买卖"""

    Buy = 0
    """买"""
    Sell = 1
    """卖"""

    def __int__(self):
        return self.value


class OffsetType(Enum):
    """开平(今)"""

    Open = 0
    """开仓"""
    Close = 1
    """平仓"""
    CloseToday = 2
    """平今
    上期所独有"""

    def __int__(self):
        return self.value


class OrderType(Enum):
    """委托类型"""

    Limit = 0
    """限价单"""
    Market = 1
    """市价单"""
    FAK = 2
    """部成立撤"""
    FOK = 3
    """全成立撤"""

    def __int__(self):
        return self.value


class OrderStatus(Enum):
    """委托状态"""

    Normal = 0
    """正常"""
    Partial = 1
    """部分成交"""
    Filled = 2
    """全部成交"""
    Canceled = 3
    """撤单"""
    Error = 4
    """错单"""

    def __int__(self):
        return self.value


class InstrumentStatus(Enum):
    """交易状态"""

    Continous = 0
    """交易"""
    Auction = 1
    """竞价"""
    NoTrading = 2
    """非交易"""
    Closed = 3
    """收盘"""

    def __int__(self):
        return self.value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : HaiFeng
# @Email   : 24918700@qq.com
# @Time    : 2022/09/11


import os
import platform
import copy

from ctypes import *
from .structs import *


class Quote:

    def __init__(self):
        dllpath = os.path.join(os.path.abspath(os.path.dirname(__file__)), "lib64")
        absolute_dllfile_path = os.path.join(
            dllpath,
            "ctp_quote.dll" if "Windows" in platform.system() else "libctp_quote.so",
        )
        if not os.path.exists(absolute_dllfile_path):
            print("缺少DLL接口文件")
            return

        # make log dir for api log
        logdir = os.path.join(os.getcwd(), "log")
        if not os.path.exists(logdir):
            os.mkdir(logdir)

        dlldir = os.path.split(absolute_dllfile_path)[0]
        # change work directory
        cur_path = os.getcwd()
        os.chdir(dlldir)

        self.h = CDLL(absolute_dllfile_path)

        self.h.qCreateApi.argtypes = []
        self.h.qCreateApi.restype = c_void_p

        self.h.qCreateSpi.argtypes = []
        self.h.qCreateSpi.restype = c_void_p

        self.api = None
        self.spi = None

        #################### 请求函数 #######################
        # 创建MdApi
        self.h.qRelease.argtypes = [c_void_p]
        self.h.qRelease.restype = c_void_p
        # 初始化
        self.h.qInit.argtypes = [c_void_p]
        self.h.qInit.restype = c_void_p
        # 等待接口线程结束运行
        self.h.qJoin.argtypes = [c_void_p]
        self.h.qJoin.restype = c_void_p
        # 注册前置机网络地址
        self.h.qRegisterFront.argtypes = [c_void_p, c_void_p]
        self.h.qRegisterFront.restype = c_void_p
        # @remark RegisterNameServer优先于RegisterFront
        self.h.qRegisterNameServer.argtypes = [c_void_p, c_void_p]
        self.h.qRegisterNameServer.restype = c_void_p
        # 注册名字服务器用户信息
        self.h.qRegisterFensUserInfo.argtypes = [c_void_p, c_void_p]
        self.h.qRegisterFensUserInfo.restype = c_void_p
        # 注册回调接口
        self.h.qRegisterSpi.argtypes = [c_void_p, c_void_p]
        self.h.qRegisterSpi.restype = c_void_p
        # 订阅行情。
        self.h.qSubscribeMarketData.argtypes = [c_void_p, c_void_p, c_int32]
        self.h.qSubscribeMarketData.restype = c_void_p
        # 退订行情。
        self.h.qUnSubscribeMarketData.argtypes = [c_void_p, c_void_p, c_int32]
        self.h.qUnSubscribeMarketData.restype = c_void_p
        # 订阅询价。
        self.h.qSubscribeForQuoteRsp.argtypes = [c_void_p, c_void_p, c_int32]
        self.h.qSubscribeForQuoteRsp.restype = c_void_p
        # 退订询价。
        self.h.qUnSubscribeForQuoteRsp.argtypes = [c_void_p, c_void_p, c_int32]
        self.h.qUnSubscribeForQuoteRsp.restype = c_void_p
        # 用户登录请求
        self.h.qReqUserLogin.argtypes = [c_void_p, c_void_p, c_int32]
        self.h.qReqUserLogin.restype = c_void_p
        # 登出请求
        self.h.qReqUserLogout.argtypes = [c_void_p, c_void_p, c_int32]
        self.h.qReqUserLogout.restype = c_void_p
        # 请求查询组播合约
        self.h.qReqQryMulticastInstrument.argtypes = [c_void_p, c_void_p, c_int32]
        self.h.qReqQryMulticastInstrument.restype = c_void_p

        os.chdir(cur_path)

    def CreateApi(self):
        self.api = self.h.qCreateApi()
        return self.api

    def CreateSpi(self):
        self.spi = self.h.qCreateSpi()
        #################### 响应函数 #########################
        # 当客户端与交易后台建立起通信连接时（还未登录前），该方法被调用。
        self.h.qSetOnFrontConnected.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnFrontConnected.restype = None
        self.evOnFrontConnected = CFUNCTYPE(None)(self.__OnFrontConnected)
        self.h.qSetOnFrontConnected(self.spi, self.evOnFrontConnected)
        # 当客户端与交易后台通信连接断开时，该方法被调用。当发生这个情况后，API会自动重新连接，客户端可不做处理。
        self.h.qSetOnFrontDisconnected.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnFrontDisconnected.restype = None
        self.evOnFrontDisconnected = CFUNCTYPE(None, c_int32)(
            self.__OnFrontDisconnected
        )
        self.h.qSetOnFrontDisconnected(self.spi, self.evOnFrontDisconnected)
        # 心跳超时警告。当长时间未收到报文时，该方法被调用。
        self.h.qSetOnHeartBeatWarning.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnHeartBeatWarning.restype = None
        self.evOnHeartBeatWarning = CFUNCTYPE(None, c_int32)(self.__OnHeartBeatWarning)
        self.h.qSetOnHeartBeatWarning(self.spi, self.evOnHeartBeatWarning)
        # 登录请求响应
        self.h.qSetOnRspUserLogin.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnRspUserLogin.restype = None
        self.evOnRspUserLogin = CFUNCTYPE(
            None,
            POINTER(CThostFtdcRspUserLoginField),
            POINTER(CThostFtdcRspInfoField),
            c_int32,
            c_bool,
        )(self.__OnRspUserLogin)
        self.h.qSetOnRspUserLogin(self.spi, self.evOnRspUserLogin)
        # 登出请求响应
        self.h.qSetOnRspUserLogout.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnRspUserLogout.restype = None
        self.evOnRspUserLogout = CFUNCTYPE(
            None,
            POINTER(CThostFtdcUserLogoutField),
            POINTER(CThostFtdcRspInfoField),
            c_int32,
            c_bool,
        )(self.__OnRspUserLogout)
        self.h.qSetOnRspUserLogout(self.spi, self.evOnRspUserLogout)
        # 请求查询组播合约响应
        self.h.qSetOnRspQryMulticastInstrument.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnRspQryMulticastInstrument.restype = None
        self.evOnRspQryMulticastInstrument = CFUNCTYPE(
            None,
            POINTER(CThostFtdcMulticastInstrumentField),
            POINTER(CThostFtdcRspInfoField),
            c_int32,
            c_bool,
        )(self.__OnRspQryMulticastInstrument)
        self.h.qSetOnRspQryMulticastInstrument(
            self.spi, self.evOnRspQryMulticastInstrument
        )
        # 错误应答
        self.h.qSetOnRspError.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnRspError.restype = None
        self.evOnRspError = CFUNCTYPE(
            None, POINTER(CThostFtdcRspInfoField), c_int32, c_bool
        )(self.__OnRspError)
        self.h.qSetOnRspError(self.spi, self.evOnRspError)
        # 订阅行情应答
        self.h.qSetOnRspSubMarketData.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnRspSubMarketData.restype = None
        self.evOnRspSubMarketData = CFUNCTYPE(
            None,
            POINTER(CThostFtdcSpecificInstrumentField),
            POINTER(CThostFtdcRspInfoField),
            c_int32,
            c_bool,
        )(self.__OnRspSubMarketData)
        self.h.qSetOnRspSubMarketData(self.spi, self.evOnRspSubMarketData)
        # 取消订阅行情应答
        self.h.qSetOnRspUnSubMarketData.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnRspUnSubMarketData.restype = None
        self.evOnRspUnSubMarketData = CFUNCTYPE(
            None,
            POINTER(CThostFtdcSpecificInstrumentField),
            POINTER(CThostFtdcRspInfoField),
            c_int32,
            c_bool,
        )(self.__OnRspUnSubMarketData)
        self.h.qSetOnRspUnSubMarketData(self.spi, self.evOnRspUnSubMarketData)
        # 订阅询价应答
        self.h.qSetOnRspSubForQuoteRsp.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnRspSubForQuoteRsp.restype = None
        self.evOnRspSubForQuoteRsp = CFUNCTYPE(
            None,
            POINTER(CThostFtdcSpecificInstrumentField),
            POINTER(CThostFtdcRspInfoField),
            c_int32,
            c_bool,
        )(self.__OnRspSubForQuoteRsp)
        self.h.qSetOnRspSubForQuoteRsp(self.spi, self.evOnRspSubForQuoteRsp)
        # 取消订阅询价应答
        self.h.qSetOnRspUnSubForQuoteRsp.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnRspUnSubForQuoteRsp.restype = None
        self.evOnRspUnSubForQuoteRsp = CFUNCTYPE(
            None,
            POINTER(CThostFtdcSpecificInstrumentField),
            POINTER(CThostFtdcRspInfoField),
            c_int32,
            c_bool,
        )(self.__OnRspUnSubForQuoteRsp)
        self.h.qSetOnRspUnSubForQuoteRsp(self.spi, self.evOnRspUnSubForQuoteRsp)
        # 深度行情通知
        self.h.qSetOnRtnDepthMarketData.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnRtnDepthMarketData.restype = None
        self.evOnRtnDepthMarketData = CFUNCTYPE(
            None, POINTER(CThostFtdcDepthMarketDataField)
        )(self.__OnRtnDepthMarketData)
        self.h.qSetOnRtnDepthMarketData(self.spi, self.evOnRtnDepthMarketData)
        # 询价通知
        self.h.qSetOnRtnForQuoteRsp.argtypes = [c_void_p, c_void_p]
        self.h.qSetOnRtnForQuoteRsp.restype = None
        self.evOnRtnForQuoteRsp = CFUNCTYPE(None, POINTER(CThostFtdcForQuoteRspField))(
            self.__OnRtnForQuoteRsp
        )
        self.h.qSetOnRtnForQuoteRsp(self.spi, self.evOnRtnForQuoteRsp)

        return self.spi

    #################### 请求函数 #######################

    def Release(self):
        """创建MdApi"""
        self.h.qRelease(self.api)

    def Init(self):
        """初始化"""
        self.h.qInit(self.api)

    def Join(self):
        """等待接口线程结束运行"""
        self.h.qJoin(self.api)

    def RegisterFront(self, pszFrontAddress: str):
        """注册前置机网络地址"""
        self.h.qRegisterFront(self.api, bytes(pszFrontAddress, encoding="ascii"))

    def RegisterNameServer(self, pszNsAddress: str):
        """@remark RegisterNameServer优先于RegisterFront"""
        self.h.qRegisterNameServer(self.api, bytes(pszNsAddress, encoding="ascii"))

    def RegisterFensUserInfo(self, pFensUserInfo: CThostFtdcFensUserInfoField):
        """注册名字服务器用户信息"""
        self.h.qRegisterFensUserInfo(self.api, byref(pFensUserInfo))

    def RegisterSpi(self, pSpi: c_void_p):
        """注册回调接口"""
        self.h.qRegisterSpi(self.api, pSpi)

    def SubscribeMarketData(self, ppInstrumentID: str, nCount: int):
        """订阅行情。"""
        self.h.qSubscribeMarketData(self.api, ppInstrumentID, nCount)

    def UnSubscribeMarketData(self, ppInstrumentID: str, nCount: int):
        """退订行情。"""
        self.h.qUnSubscribeMarketData(self.api, ppInstrumentID, nCount)

    def SubscribeForQuoteRsp(self, ppInstrumentID: str, nCount: int):
        """订阅询价。"""
        self.h.qSubscribeForQuoteRsp(self.api, ppInstrumentID, nCount)

    def UnSubscribeForQuoteRsp(self, ppInstrumentID: str, nCount: int):
        """退订询价。"""
        self.h.qUnSubscribeForQuoteRsp(self.api, ppInstrumentID, nCount)

    def ReqUserLogin(
        self, pReqUserLoginField: CThostFtdcReqUserLoginField, nRequestID: int
    ):
        """用户登录请求"""
        self.h.qReqUserLogin(self.api, byref(pReqUserLoginField), nRequestID)

    def ReqUserLogout(self, pUserLogout: CThostFtdcUserLogoutField, nRequestID: int):
        """登出请求"""
        self.h.qReqUserLogout(self.api, byref(pUserLogout), nRequestID)

    def ReqQryMulticastInstrument(
        self,
        pQryMulticastInstrument: CThostFtdcQryMulticastInstrumentField,
        nRequestID: int,
    ):
        """请求查询组播合约"""
        self.h.qReqQryMulticastInstrument(
            self.api, byref(pQryMulticastInstrument), nRequestID
        )

    #################### 响应函数 #########################

    def __OnFrontConnected(self):
        self.OnFrontConnected()

    def OnFrontConnected(self):
        """当客户端与交易后台建立起通信连接时（还未登录前），该方法被调用。"""
        print("===OnFrontConnected===:")

    def __OnFrontDisconnected(self, nReason: int):
        self.OnFrontDisconnected(nReason)

    def OnFrontDisconnected(self, nReason: int):
        """当客户端与交易后台通信连接断开时，该方法被调用。当发生这个情况后，API会自动重新连接，客户端可不做处理。"""
        print("===OnFrontDisconnected===:nReason:int")

    def __OnHeartBeatWarning(self, nTimeLapse: int):
        self.OnHeartBeatWarning(nTimeLapse)

    def OnHeartBeatWarning(self, nTimeLapse: int):
        """心跳超时警告。当长时间未收到报文时，该方法被调用。"""
        print("===OnHeartBeatWarning===:nTimeLapse:int")

    def __OnRspUserLogin(
        self,
        pRspUserLogin: CThostFtdcRspUserLoginField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        self.OnRspUserLogin(
            (
                copy.deepcopy(
                    POINTER(CThostFtdcRspUserLoginField)
                    .from_param(pRspUserLogin)
                    .contents
                )
                if pRspUserLogin
                else CThostFtdcRspUserLoginField()
            ),
            (
                copy.deepcopy(
                    POINTER(CThostFtdcRspInfoField).from_param(pRspInfo).contents
                )
                if pRspInfo
                else CThostFtdcRspInfoField()
            ),
            nRequestID,
            bIsLast,
        )

    def OnRspUserLogin(
        self,
        pRspUserLogin: CThostFtdcRspUserLoginField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """登录请求响应"""
        print(
            "===OnRspUserLogin===:pRspUserLogin:CThostFtdcRspUserLoginField, pRspInfo:CThostFtdcRspInfoField, nRequestID:int, bIsLast:bool"
        )

    def __OnRspUserLogout(
        self,
        pUserLogout: CThostFtdcUserLogoutField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        self.OnRspUserLogout(
            (
                copy.deepcopy(
                    POINTER(CThostFtdcUserLogoutField).from_param(pUserLogout).contents
                )
                if pUserLogout
                else CThostFtdcUserLogoutField()
            ),
            (
                copy.deepcopy(
                    POINTER(CThostFtdcRspInfoField).from_param(pRspInfo).contents
                )
                if pRspInfo
                else CThostFtdcRspInfoField()
            ),
            nRequestID,
            bIsLast,
        )

    def OnRspUserLogout(
        self,
        pUserLogout: CThostFtdcUserLogoutField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """登出请求响应"""
        print(
            "===OnRspUserLogout===:pUserLogout:CThostFtdcUserLogoutField, pRspInfo:CThostFtdcRspInfoField, nRequestID:int, bIsLast:bool"
        )

    def __OnRspQryMulticastInstrument(
        self,
        pMulticastInstrument: CThostFtdcMulticastInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        self.OnRspQryMulticastInstrument(
            (
                copy.deepcopy(
                    POINTER(CThostFtdcMulticastInstrumentField)
                    .from_param(pMulticastInstrument)
                    .contents
                )
                if pMulticastInstrument
                else CThostFtdcMulticastInstrumentField()
            ),
            (
                copy.deepcopy(
                    POINTER(CThostFtdcRspInfoField).from_param(pRspInfo).contents
                )
                if pRspInfo
                else CThostFtdcRspInfoField()
            ),
            nRequestID,
            bIsLast,
        )

    def OnRspQryMulticastInstrument(
        self,
        pMulticastInstrument: CThostFtdcMulticastInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """请求查询组播合约响应"""
        print(
            "===OnRspQryMulticastInstrument===:pMulticastInstrument:CThostFtdcMulticastInstrumentField, pRspInfo:CThostFtdcRspInfoField, nRequestID:int, bIsLast:bool"
        )

    def __OnRspError(
        self, pRspInfo: CThostFtdcRspInfoField, nRequestID: int, bIsLast: bool
    ):
        self.OnRspError(
            (
                copy.deepcopy(
                    POINTER(CThostFtdcRspInfoField).from_param(pRspInfo).contents
                )
                if pRspInfo
                else CThostFtdcRspInfoField()
            ),
            nRequestID,
            bIsLast,
        )

    def OnRspError(
        self, pRspInfo: CThostFtdcRspInfoField, nRequestID: int, bIsLast: bool
    ):
        """错误应答"""
        print(
            "===OnRspError===:pRspInfo:CThostFtdcRspInfoField, nRequestID:int, bIsLast:bool"
        )

    def __OnRspSubMarketData(
        self,
        pSpecificInstrument: CThostFtdcSpecificInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        self.OnRspSubMarketData(
            (
                copy.deepcopy(
                    POINTER(CThostFtdcSpecificInstrumentField)
                    .from_param(pSpecificInstrument)
                    .contents
                )
                if pSpecificInstrument
                else CThostFtdcSpecificInstrumentField()
            ),
            (
                copy.deepcopy(
                    POINTER(CThostFtdcRspInfoField).from_param(pRspInfo).contents
                )
                if pRspInfo
                else CThostFtdcRspInfoField()
            ),
            nRequestID,
            bIsLast,
        )

    def OnRspSubMarketData(
        self,
        pSpecificInstrument: CThostFtdcSpecificInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """订阅行情应答"""
        print(
            "===OnRspSubMarketData===:pSpecificInstrument:CThostFtdcSpecificInstrumentField, pRspInfo:CThostFtdcRspInfoField, nRequestID:int, bIsLast:bool"
        )

    def __OnRspUnSubMarketData(
        self,
        pSpecificInstrument: CThostFtdcSpecificInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        self.OnRspUnSubMarketData(
            (
                copy.deepcopy(
                    POINTER(CThostFtdcSpecificInstrumentField)
                    .from_param(pSpecificInstrument)
                    .contents
                )
                if pSpecificInstrument
                else CThostFtdcSpecificInstrumentField()
            ),
            (
                copy.deepcopy(
                    POINTER(CThostFtdcRspInfoField).from_param(pRspInfo).contents
                )
                if pRspInfo
                else CThostFtdcRspInfoField()
            ),
            nRequestID,
            bIsLast,
        )

    def OnRspUnSubMarketData(
        self,
        pSpecificInstrument: CThostFtdcSpecificInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """取消订阅行情应答"""
        print(
            "===OnRspUnSubMarketData===:pSpecificInstrument:CThostFtdcSpecificInstrumentField, pRspInfo:CThostFtdcRspInfoField, nRequestID:int, bIsLast:bool"
        )

    def __OnRspSubForQuoteRsp(
        self,
        pSpecificInstrument: CThostFtdcSpecificInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        self.OnRspSubForQuoteRsp(
            (
                copy.deepcopy(
                    POINTER(CThostFtdcSpecificInstrumentField)
                    .from_param(pSpecificInstrument)
                    .contents
                )
                if pSpecificInstrument
                else CThostFtdcSpecificInstrumentField()
            ),
            (
                copy.deepcopy(
                    POINTER(CThostFtdcRspInfoField).from_param(pRspInfo).contents
                )
                if pRspInfo
                else CThostFtdcRspInfoField()
            ),
            nRequestID,
            bIsLast,
        )

    def OnRspSubForQuoteRsp(
        self,
        pSpecificInstrument: CThostFtdcSpecificInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """订阅询价应答"""
        print(
            "===OnRspSubForQuoteRsp===:pSpecificInstrument:CThostFtdcSpecificInstrumentField, pRspInfo:CThostFtdcRspInfoField, nRequestID:int, bIsLast:bool"
        )

    def __OnRspUnSubForQuoteRsp(
        self,
        pSpecificInstrument: CThostFtdcSpecificInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        self.OnRspUnSubForQuoteRsp(
            (
                copy.deepcopy(
                    POINTER(CThostFtdcSpecificInstrumentField)
                    .from_param(pSpecificInstrument)
                    .contents
                )
                if pSpecificInstrument
                else CThostFtdcSpecificInstrumentField()
            ),
            (
                copy.deepcopy(
                    POINTER(CThostFtdcRspInfoField).from_param(pRspInfo).contents
                )
                if pRspInfo
                else CThostFtdcRspInfoField()
            ),
            nRequestID,
            bIsLast,
        )

    def OnRspUnSubForQuoteRsp(
        self,
        pSpecificInstrument: CThostFtdcSpecificInstrumentField,
        pRspInfo: CThostFtdcRspInfoField,
        nRequestID: int,
        bIsLast: bool,
    ):
        """取消订阅询价应答"""
        print(
            "===OnRspUnSubForQuoteRsp===:pSpecificInstrument:CThostFtdcSpecificInstrumentField, pRspInfo:CThostFtdcRspInfoField, nRequestID:int, bIsLast:bool"
        )

    def __OnRtnDepthMarketData(self, pDepthMarketData: CThostFtdcDepthMarketDataField):
        self.OnRtnDepthMarketData(
            copy.deepcopy(
                POINTER(CThostFtdcDepthMarketDataField)
                .from_param(pDepthMarketData)
                .contents
            )
            if pDepthMarketData
            else CThostFtdcDepthMarketDataField()
        )

    def OnRtnDepthMarketData(self, pDepthMarketData: CThostFtdcDepthMarketDataField):
        """深度行情通知"""
        print(
            "===OnRtnDepthMarketData===:pDepthMarketData:CThostFtdcDepthMarketDataField"
        )

    def __OnRtnForQuoteRsp(self, pForQuoteRsp: CThostFtdcForQuoteRspField):
        self.OnRtnForQuoteRsp(
            copy.deepcopy(
                POINTER(CThostFtdcForQuoteRspField).from_param(pForQuoteRsp).contents
            )
            if pForQuoteRsp
            else CThostFtdcForQuoteRspField()
        )

    def OnRtnForQuoteRsp(self, pForQuoteRsp: CThostFtdcForQuoteRspField):
        """询价通知"""
        print("===OnRtnForQuoteRsp===:pForQuoteRsp:CThostFtdcForQuoteRspField")



from setuptools import setup
import os
from os import path as os_path

this_directory = os_path.abspath(os_path.dirname(__file__))


# 读取文件内容
def read_file(filename):
    desc = ""
    with open(os_path.join(this_directory, filename), encoding="utf-8") as f:
        desc = f.read()
    return desc


# 获取依赖
def read_requirements(filename):
    return [line.strip() for line in read_file(filename).splitlines() if not line.startswith("#")]


# 删除无用文件
path = f"./py_ctp/lib64"
for f in os.listdir(path):
    if os.path.isdir(f):
        continue
    if os.path.splitext(f)[1] not in [".dll", ".so"]:
        os.remove(f"./py_ctp/lib64/{f}")

long_description = read_file("setup.md")

# 虚拟环境
#   python -m venv tutorial-env
# source tutorial-env/bin/activate
# 安装信赖
#   pip install --upgrade setuptools wheel keyring twine pipreqs
# 生成 requirements.txt
#   pipreqs --encoding=utf8 --force py_ctp
# 安装 .tar.gz
#   rm dist -rf && python setup.py sdist && twine upload -u haifengat dist/*
# 安装 .whl
#   rm dist -rf && python setup.py bdist_wheel && twine upload -u haifengat dist/*.whl
#
setup(
    name="py_ctp",  # 包名
    python_requires=">=3.6.0",  # python环境
    version="6.7.2.20251003",
    description="Python CTP futures api",  # 包简介，显示在PyPI上
    long_description=long_description,  # 读取的Readme文档内容
    long_description_content_type="text/markdown",  # 指定包文档格式为markdown
    author="HaiFengAT",  # 作者相关信息
    author_email="haifengat@vip.qq.com",
    url="https://github.com/haifengat/pyctp",
    # library_dirs = ['/usr/lib'],
    # extra_link_args.append（'-Wl，-rpath，'+ lib_path）
    # 指定包信息，还可以用find_packages()函数
    # packages=find_packages(),
    packages=["py_ctp"],
    install_requires=read_requirements("requirements.txt"),  # 指定需要安装的依赖
    include_package_data=True,
    license="MIT License",
    platforms="any",
    data_files=["README.md", ("/usr/lib/py_ctp", ["py_ctp/lib64/thostmduserapi_se.so", "py_ctp/lib64/thosttraderapi_se.so"])],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
__title__ = ''
__author__ = 'HaiFeng'
__mtime__ = '2016/9/21'
"""

from .enums import DirectType, OffsetType, OrderStatus, HedgeType, TradeTypeType


class InfoField:
    """返回信息"""

    def __init__(self):
        """Constructor"""
        """错误号"""
        self.ErrorID = 0
        """错误号"""
        """错误描述"""
        self.ErrorMsg = "正确"
        """错误描述"""

    def __str__(self):
        # return 'ErrorID:{0}, ErrorMsg:{1}'.format(self.ErrorID, self.ErrorMsg)
        return '{{"ErrorID":{self.ErrorID}, "ErrorMsg":"{self.ErrorMsg}"}}'.format(self=self)

    @property
    def __dict__(self):
        return {"ErrorID": self.ErrorID, "ErrorMsg": str(self.ErrorMsg, "GB2312")}


class OrderField:
    """报单响应"""

    def __init__(self):
        """initionalize"""
        """委托标识"""
        self.OrderID = ""
        """委托标识"""
        """合约"""
        self.InstrumentID = ""
        """合约"""
        """交易所"""
        self.ExchangeID = ""
        """交易所"""
        """买卖"""
        self.Direction = DirectType.Buy
        """买卖"""
        """开平"""
        self.Offset = OffsetType.Open
        """开平"""
        """限价单价格"""
        self.LimitPrice = 0.0
        """限价单价格"""
        """报单均价"""
        self.AvgPrice = 0.0
        """报单均价"""
        """委托时间"""
        self.InsertTime = ""
        """委托时间"""
        """成交时间"""
        self.TradeTime = ""
        """成交时间"""
        """成交数量(本次)"""
        self.TradeVolume = 0
        """成交数量(本次)"""
        """委托数量"""
        self.Volume = 0
        """委托数量"""
        """未成交数量"""
        self.VolumeLeft = 0
        """未成交数量"""
        """委托状态"""
        self.Status = OrderStatus.Normal
        """委托状态"""
        """状态描述"""
        self.StatusMsg = ""
        """状态描述"""
        """是否本地委托"""
        self.IsLocal = False
        """是否本地委托"""
        """委托自定义标识"""
        self.Custom = 0
        """委托自定义标识"""
        """系统(交易所)ID"""
        self.SysID = ""
        """系统(交易所)ID"""

    def __str__(self):
        """"""
        return "{self.OrderID}, {self.InstrumentID}, {self.ExchangeID}, {self.Direction}, {self.Offset}, {self.LimitPrice}, {self.AvgPrice}, {self.InsertTime}, {self.TradeTime}, {self.TradeVolume}, {self.Volume}, {self.VolumeLeft}, {self.Status}, {self.StatusMsg}, {self.IsLocal}, {self.Custom}, {self.SysID}".format(
            self=self
        )

    @property
    def __dict__(self):  # 如何控制dict的字段次序?:交由客户端处理
        return {
            "OrderID": self.OrderID,
            "InstrumentID": self.InstrumentID,
            "ExchangeID": self.ExchangeID,
            "Direction": self.Direction.name,
            "Offset": self.Offset.name,
            "LimitPrice": self.LimitPrice,
            "AvgPrice": self.AvgPrice,
            "InsertTime": self.InsertTime,
            "TradeTime": self.TradeTime,
            "TradeVolume": self.TradeVolume,
            "Volume": self.Volume,
            "VolumeLeft": self.VolumeLeft,
            "Status": self.Status.name,
            "StatusMsg": self.StatusMsg,
            "IsLocal": self.IsLocal,
            "Custom": self.Custom,
            "SysID": self.SysID,
        }


class TradeField:
    """成交响应"""

    def __init__(self):
        """Constructor"""
        """成交标识"""
        self.TradeID = ""
        """成交标识"""
        """合约"""
        self.InstrumentID = ""
        """合约"""
        """交易所"""
        self.ExchangeID = ""
        """交易所"""
        """买卖"""
        self.Direction = DirectType.Buy
        """买卖"""
        """开平"""
        self.Offset = OffsetType.Open
        """开平"""
        """成交价"""
        self.Price = 0.0
        """成交价"""
        """成交数量"""
        self.Volume = 0
        """成交数量"""
        """成交时间"""
        self.TradeTime = ""
        """成交时间"""
        """交易日"""
        self.TradingDay = ""
        """交易日"""
        """'对应的委托标识"""
        self.OrderID = ""
        """'对应的委托标识"""
        """对应的系统(交易所)ID"""
        self.SysID = ""
        """对应的系统(交易所)ID"""

    def __str__(self):
        """"""
        return "{self.TradeID}, {self.InstrumentID}, {self.ExchangeID}, {self.Direction}, {self.Offset}, {self.Price}, {self.Volume}, {self.TradeTime}, {self.TradingDay}, {self.OrderID}, {self.SysID}".format(
            self=self
        )

    @property
    def __dict__(self):
        return {
            "TradeID": self.TradeID,
            "InstrumentID": self.InstrumentID,
            "ExchangeID": self.ExchangeID,
            "Direction": self.Direction.name,
            "Offset": self.Offset.name,
            "Price": self.Price,
            "Volume": self.Volume,
            "TradeTime": self.TradeTime,
            "TradingDay": self.TradingDay,
            "OrderID": self.OrderID,
            "SysID": self.SysID,
        }


class InstrumentField:
    """合约"""

    def __init__(self):
        """Constructor"""
        """合约"""
        self.InstrumentID = ""
        """合约"""
        """名称"""
        self.InstrumentName = ""
        """名称"""
        """品种"""
        self.ProductID = ""
        """品种"""
        """交易所"""
        self.ExchangeID = ""
        """交易所"""
        """合约乘数"""
        self.VolumeMultiple = ""
        """合约乘数"""
        """每跳价格变动"""
        self.PriceTick = 0.0
        """每跳价格变动"""
        """最大单笔下单量"""
        self.MaxOrderVolume = 9999
        """最大单笔下单量"""
        """类型 Futures/Options/Combination/Spot/EFP/SpotOption"""
        self.ProductType = "Future"
        """类型 Futures/Options/Combination/Spot/EFP/SpotOption"""

    def __str__(self):
        """"""
        return "{self.InstrumentID}, {self.ProductID}, {self.ExchangeID}, {self.VolumeMultiple}, {self.PriceTick}, {self.MaxOrderVolume}, {self.ProductType}".format(
            self=self
        )

    @property
    def __dict__(self):
        return {
            "InstrumentID": self.InstrumentID,
            "ProductID": self.ProductID,
            "ExchangeID": self.ExchangeID,
            "VolumeMultiple": self.VolumeMultiple,
            "PriceTick": self.PriceTick,
            "MaxOrderVolume": self.MaxOrderVolume,
            "ProductType": self.ProductType,
        }


class TradingAccount:
    """交易帐户"""

    def __init__(self):
        """Constructor"""
        """昨日结算"""
        self.PreBalance = 0.0
        """昨日结算"""
        """持仓盈亏"""
        self.PositionProfit = 0.0
        """持仓盈亏"""
        """平仓盈亏"""
        self.CloseProfit = 0.0
        """平仓盈亏"""
        """手续费"""
        self.Commission = 0.0
        """手续费"""
        """保证金"""
        self.CurrMargin = 0.0
        """保证金"""
        """冻结"""
        self.FrozenCash = 0.0
        """冻结"""
        """可用"""
        self.Available = 0.0
        """可用"""
        """动态权益"""
        self.Fund = 0.0
        """动态权益"""
        """风险度"""
        self.Risk = 0.0
        """风险度"""

    # ----------------------------------------------------------------------
    def __str__(self):
        """"""
        return "{self.PreBalance}, {self.PositionProfit}, {self.CloseProfit}, {self.Commission}, {self.CurrMargin}, {self.FrozenCash}, {self.Available}, {self.Fund}, {self.Risk}".format(
            self=self
        )

    @property
    def __dict__(self):
        return {
            "PreBalance": self.PreBalance,
            "PositionProfit": self.PositionProfit,
            "CloseProfit": self.CloseProfit,
            "Commission": self.Commission,
            "CurrMargin": self.CurrMargin,
            "FrozenCash": self.FrozenCash,
            "Available": self.Available,
            "Fund": self.Fund,
            "Risk": self.Risk,
        }


########################################################################
class PositionField:
    """持仓"""

    # ----------------------------------------------------------------------
    def __init__(self):
        """Constructor"""
        """合约"""
        self.InstrumentID = ""
        """合约"""
        self.Direction = DirectType.Buy
        """多空"""
        """持仓价格"""
        self.Price = 0.0
        """持仓价格"""
        """持仓量"""
        self.Position = 0
        """持仓量"""
        """昨持仓"""
        self.YdPosition = 0
        """昨持仓"""
        """今持仓"""
        self.TdPosition = 0
        """今持仓"""
        """平仓盈亏"""
        self.CloseProfit = 0.0
        """平仓盈亏"""
        """持仓盈亏"""
        self.PositionProfit = 0.0
        """持仓盈亏"""
        """手续费"""
        self.Commission = 0.0
        """手续费"""
        """保证金"""
        self.Margin = 0.0
        """保证金"""

    # ----------------------------------------------------------------------
    def __str__(self):
        """"""
        return (
            "{self.InstrumentID}, {self.Direction}, {self.Price}, {self.Position}, {self.TdPosition}, {self.YdPosition}, {self.CloseProfit}, {self.PositionProfit}, {self.Commission}, {self.Margin}"
        ).format(self=self)

    @property
    def __dict__(self):
        return {
            "InstrumentID": self.InstrumentID,
            "Direction": self.Direction.name,
            "Price": self.Price,
            "Position": self.Position,
            "TdPosition": self.TdPosition,
            "YdPosition": self.YdPosition,
            "CloseProfit": self.CloseProfit,
            "PositionProfit": self.PositionProfit,
            "Commission": self.Commission,
            "Margin": self.Margin,
        }


class PositionDetail:
    """持仓明细"""

    def __init__(self):
        """"""
        self.Instrument = ""
        """合约"""
        self.HedgeFlag = HedgeType.Speculation
        """投保"""
        self.Direction = DirectType.Buy
        """持仓方向"""
        self.TradeID = ""
        """交易ID"""
        self.Volume = 0
        """持仓量"""
        self.OpenPrice = 0.0
        """平仓价格"""
        self.OpenDate = ""
        """开仓日期"""
        self.TradeType = TradeTypeType.Common
        """交易类型"""
        self.PositionProfit = 0.0
        """盯市持仓盈亏"""
        self.CloseProfit = 0.0
        """盯市平仓盈亏"""


class Tick:
    """分笔数据"""

    def __init__(self):
        """初始化"""
        """合约"""
        self.Instrument = ""
        """合约"""
        """最新价"""
        self.LastPrice = 0.0
        """最新价"""
        """挂卖价"""
        self.AskPrice = 0.0
        """挂卖价"""
        """挂买价"""
        self.BidPrice = 0.0
        """挂买价"""
        """挂卖量"""
        self.AskVolume = 1
        """挂卖量"""
        """挂买量"""
        self.BidVolume = 1
        """挂买量"""
        """时间"""
        self.UpdateTime = ""
        """时间"""
        """毫秒"""
        self.UpdateMillisec = 0
        """毫秒"""
        """成交量"""
        self.Volume = 1
        """成交量"""
        """持仓量"""
        self.OpenInterest = 1.0
        """持仓量"""
        """均价"""
        self.AveragePrice = 0.0
        """均价"""
        """涨板价"""
        self.UpperLimitPrice = 0.0
        """涨板价"""
        """跌板价"""
        self.LowerLimitPrice = 0.0
        """跌板价"""
        """昨结算价"""
        self.PreOpenInterest = 0.0

    def __str__(self):
        """"""
        return "{self.Instrument}, {self.LastPrice}, {self.AskPrice}, {self.AskVolume}, {self.BidPrice}, {self.BidVolume}, {self.UpdateTime}, {self.Volume}, {self.OpenInterest}, {self.AveragePrice}, {self.UpperLimitPrice}, {self.LowerLimitPrice}, {self.PreOpenInterest}".format(
            self=self
        )

    # @property
    # def __dict__(self):
    #     return {
    #         'Instrument': self.Instrument,
    #         'LastPrice': self.LastPrice,
    #         'AskPrice': self.AskPrice,
    #         'AskVolume': self.AskVolume,
    #         'BidPrice': self.BidPrice,
    #         'BidVolume': self.BidVolume,
    #         'UpdateTime': self.UpdateTime,
    #         'Volume': self.Volume,
    #         'OpenInterest': self.OpenInterest,
    #         'AveragePrice': self.AveragePrice
    #     }
